from pathlib import Path

import numpy as np

import cmac_gdal_io as io
import cmac_utility as util



def create_nodata_mask_buffer(input_image_buffer,
                              nodata_value):
    
    # Allocate the data buffer.
    buff_ysize = input_image_buffer.shape[0]
    buff_xsize = input_image_buffer.shape[1]

    mask_image_buffer = (input_image_buffer != nodata_value).astype(input_image_buffer.dtype)

    return mask_image_buffer




def create_nodata_mask_images(toa_datasets_list,
                              bands_to_process_list,
                              nodata_value,
                              proj_info_string,
                              geotransform_info,
                              GTiff_mask_output_path,
                              image_basename):

    num_toa_datasets = len(toa_datasets_list)

    final_status = 0
    for index, ds in enumerate(toa_datasets_list):
    
       # First, make sure a mask file hasn't already been
       # created for this band.
       mask_fname = GTiff_mask_output_path + '/' + image_basename \
           + f'_{bands_to_process_list[index]}_nodata_mask.tif'
       if Path(mask_fname).exists():
           print(f'Band {bands_to_process_list[index]} mask image ' \
               + 'has already been created, skipping')
           continue

       print(f'Creating band {bands_to_process_list[index]} mask image...')
       
       # Read the image data.
       image_data_buffer = io.read_image_from_open_dataset(ds)

       # Create the mask image.
       mask_data_buffer = create_nodata_mask_buffer(image_data_buffer,
                                                    nodata_value)

       # Write the mask image to file.
       output_image_basename = image_basename + f'_{bands_to_process_list[index]}_nodata_mask'
       status = io.write_image_to_geotiff(mask_data_buffer, proj_info_string,
                                          geotransform_info, GTiff_mask_output_path,
                                          output_image_basename)
 
       if status < 0:
           final_status = -1
           err_msg = 'ERROR: could not create new mask image file for ' \
               + f'"{GTiff_mask_filenames_list[i]}" and/or ' \
               + f'"{product_mask_filenames_list[i]}"'
           print(err_msg)

    return final_status
