from bs4 import BeautifulSoup
import numpy as np


def get_bsoup_data(bsoup_data, label):
    label_data = bsoup_data.find(label)
    return label_data



def get_solar_zenith_angle(metadata_filename):

    # Read in the (XML-formatted) metadata file contents.
    with open(metadata_filename, 'r') as fid:
        metadata = fid.read()
        bsoup_metadata = BeautifulSoup(metadata, "xml")

    # Get the solar elevation angle info. The zenith
    # angle is (90 - solar elevation).
    solar_zenith \
        = 90.0 - np.float(get_bsoup_data(bsoup_metadata, "SUN_ELEVATION").string)

    return solar_zenith




def get_band_reflectance_conversion_factors(metadata_filename,
                                            bands_to_process):

    # Read in the (XML-formatted) metadata file contents.
    with open(metadata_filename, 'r') as fid:
        metadata = fid.read()
        bsoup_metadata = BeautifulSoup(metadata, "xml")

    # Set up the list to contain the conversion factors for
    # each specified band as tuples. Tuple element #1 is the
    # band ID, tuple element #2 is the multiplicative factor,
    # and tuple element #3 is the additive factor.
    conversion_factor_list = []

    for band in bands_to_process:
        band_number = band[1:]
        reflectance_mult_label = f'REFLECTANCE_MULT_BAND_{band_number}'
        reflectance_add_label = f'REFLECTANCE_ADD_BAND_{band_number}'

        mult_factor \
            = np.float(get_bsoup_data(bsoup_metadata, reflectance_mult_label).string)
        add_factor \
            = np.float(get_bsoup_data(bsoup_metadata, reflectance_add_label).string)
        conversion_factor_list.append((band, mult_factor, add_factor))

    return conversion_factor_list
