import json
from pathlib import Path
import shutil


# Global variable.
l89_band_id_master_list = ['B2', 'B3', 'B4', 'B5', 'B6', 'B7']



# Given an index number into the band ID master list, get the
# corresponding band ID string.
def get_band_ids_list(band_indices_list):

    # How many bands do we need to find?
    num_bands_to_find = len(band_indices_list)

    band_ids_list = []
    num_bands_found = 0
    for index in band_indices_list:
        band_id = ''
        if index == 0:
            band_id = 'B2'
        elif index == 1:
            band_id = 'B3'
        elif index == 2:
            band_id = 'B4'
        elif index == 3:
            band_id = 'B5'
        elif index == 4:
            band_id = 'B6'
        elif index == 5:
            band_id = 'B7'
        else:
            err_msg = f'ERROR: band ID corresponding to index value {index} ' \
                + 'not found'
            print(err_msg)
            continue
        num_bands_found += 1
        band_ids_list.append(band_id)

    # Make sure we've found all the specified bands.
    if num_bands_found != num_bands_to_find:
        err_msg = f'ERROR: {num_bands_found} bands found, but ' \
            + f'{num_bands_to_find} bands are needed'
        print(err_msg)
        return []

    return band_ids_list




# Function to construct partial or total absolute paths for image
# data files.
def construct_image_filenames(parent_path, image_basename,
                              prefix, suffix, bands_to_process_list,
                              file_ext):

    # Initializations.
    image_filenames_list = []
    num_bands_to_process = len(bands_to_process_list)
    num_filenames_constructed = 0

    for band in bands_to_process_list:

        curr_filename = parent_path + '/'
        if prefix != '':
            curr_filename = curr_filename + prefix + '_'
        curr_filename = curr_filename + image_basename

        curr_filename = curr_filename + '_' + band
        if suffix != '':
            curr_filename = curr_filename + '_' + suffix
        curr_filename = curr_filename + f'.{file_ext}'
       
        # Add this filename to the list.
        image_filenames_list.append(curr_filename)
        num_filenames_constructed += 1

    # Did we create all the filenames we needed to?
    if num_filenames_constructed != num_bands_to_process:
        err_msg = f'ERROR: {num_filenames_constructed} image filenames ' \
            + f'constructed, but {num_bands_to_process} are required'
        print(err_msg)
        return []

    return image_filenames_list







# Polynomial evaluation using Horner's algorithm.
def eval_horner_poly(x, coeffs_list, num_coeffs):

    # Start with the highest power coefficient and work
    # backwords. Assumes polynomial coefficients are ordered
    # from const to highest powered term.
    poly_value = coeffs_list[num_coeffs-1];
    #print(poly_value)

    # End value in a python(range) interval is NOT inclusive --
    # need to add/subtract one to get ALL elements considered
    for i in range(num_coeffs-2, -1, -1):
        poly_value *= x
        poly_value += coeffs_list[i]
        #print(poly_value)

    return poly_value




# Run this at the tail end, after the report has
# been generated.
def clean_cmac_gtiffs(cmac_image_path):
    path = Path(cmac_image_path)
    if not path.exists():
        err_msg = f'{cmac_image_path} not found, cannot clean'
        print(err_msg)
        return

    # Selectively remove the GTiff image files.
    GTiff_file_list = path.glob('*.tif')
    for item in GTiff_file_list:
        fpath = Path(item)
        if fpath.is_file():
            fpath.unlink()




# Function to construct partial or total absolute paths for image
# data files.
def construct_image_filenames(parent_path, image_basename,
                              prefix, suffix, bands_to_process_list,
                              file_ext):

    # Initializations.
    image_filenames_list = []
    num_bands_to_process = len(bands_to_process_list)
    num_filenames_constructed = 0

    for band in bands_to_process_list:

        curr_filename = parent_path + '/'
        if prefix != '':
            curr_filename = curr_filename + prefix + '_'
        curr_filename = curr_filename + image_basename

        curr_filename = curr_filename + f'_{band}'
        if suffix != '':
            curr_filename = curr_filename + '_' + suffix
        curr_filename = curr_filename + f'.{file_ext}'

        # Add this filename to the list.
        image_filenames_list.append(curr_filename)
        num_filenames_constructed += 1

    # Did we create all the filenames we needed to?
    if num_filenames_constructed != num_bands_to_process:
        err_msg = f'ERROR: {num_filenames_constructed} image filenames ' \
            + f'constructed, but {num_bands_to_process} are required'
        print(err_msg)
        return []

    return image_filenames_list

